@@ -532,7 +532,7 @@ config.REPLICATION_ENABLED = true;
 ///////////////////////
 
 config.NAMESPACE_CACHING = {
-    DEFAULT_CACHE_TTL_MS: 60000,
+    DEFAULT_CACHE_TTL_MS: 0,
     DEFAULT_BLOCK_SIZE: 64 * 1024,
     DEFAULT_MAX_CACHE_OBJECT_SIZE: 4 * 1024 * 1024 * 1024 * 1024,
     DISABLE_BUCKET_FREE_SPACE_CHECK: false,
@@ -556,7 +556,6 @@ assert(config.NAMESPACE_CACHING.DEFAULT_BLOCK_SIZE > config.INLINE_MAX_SIZE);
 
 config.NSFS_BUF_SIZE = 8 * 1024 * 1024;
 config.NSFS_BUF_POOL_MEM_LIMIT = config.BUFFERS_MEM_LIMIT;
-config.NSFS_DEFAULT_IOV_MAX = 1024; // see IOV_MAX in https://man7.org/linux/man-pages/man0/limits.h.0p.html
 
 // the temporary path for uploads and other internal files
 config.NSFS_TEMP_DIR_NAME = '.noobaa-nsfs';
@@ -584,6 +583,10 @@ config.NSFS_SEM_WARNING_TIMEOUT = 10 * 60 * 1000;
 // number of rename retries in case of deleted destination directory
 config.NSFS_RENAME_RETRIES = 3;
 
+//Quota
+config.QUOTA_LOW_THRESHOLD = 80;
+config.QUOTA_MAX_OBJECTS = Number.MAX_SAFE_INTEGER;
+
 /////////////////////
 //                 //
 //    OVERRIDES    //
@@ -645,6 +648,10 @@ function load_config_env_overrides() {
                 console.log(`Overriding config.js from ENV with ${conf_name}=${val} (string)`);
                 config[conf_name] = val;
 
+            } else if (type === 'object') {
+                // TODO: Validation checks, more complex type casting for values if needed
+                config[conf_name] = Array.isArray(prev_val) ? val.split(',') : JSON.parse(val);
+                console.log(`Overriding config.js from ENV with ${conf_name}=${val} (object of type ${Array.isArray(prev_val) ? 'array' : 'json'})`);
             } else {
                 console.log(`Unknown type or mismatch between existing ${type} and provided type for ${conf_name}, skipping ...`);
             }
