@@ -10,7 +10,7 @@
  */
 module.exports = {
 
-    id: 'account_api',
+    $id: 'account_api',
 
     methods: {
 
@@ -73,6 +73,12 @@ module.exports = {
                     nsfs_account_config: {
                         $ref: 'common_api#/definitions/nsfs_account_config'
                     },
+                    access_keys: {
+                        type: 'array',
+                        items: {
+                            $ref: 'common_api#/definitions/access_keys'
+                        }
+                    },
                 },
             },
             reply: {
@@ -389,7 +395,7 @@ module.exports = {
             method: 'PUT',
             params: {
                 type: 'object',
-                required: ['name', 'identity', 'secret', 'endpoint', 'endpoint_type'],
+                required: ['name', 'endpoint', 'endpoint_type'],
                 properties: {
                     name: {
                         type: 'string'
@@ -399,6 +405,12 @@ module.exports = {
                     },
                     identity: { $ref: 'common_api#/definitions/access_key' },
                     secret: { $ref: 'common_api#/definitions/secret_key' },
+                    aws_sts_arn: {
+                        type: 'string'
+                    },
+                    region: {
+                        type: 'string'
+                    },
                     auth_method: {
                         $ref: 'common_api#/definitions/cloud_auth_method'
                     },
@@ -439,7 +451,7 @@ module.exports = {
             method: 'GET',
             params: {
                 type: 'object',
-                required: ['identity', 'secret', 'endpoint'],
+                required: ['endpoint', 'endpoint_type'],
                 properties: {
                     name: {
                         type: 'string'
@@ -449,6 +461,12 @@ module.exports = {
                     },
                     identity: { $ref: 'common_api#/definitions/access_key' },
                     secret: { $ref: 'common_api#/definitions/secret_key' },
+                    aws_sts_arn: {
+                        type: 'string'
+                    },
+                    region: {
+                        type: 'string'
+                    },
                     auth_method: {
                         $ref: 'common_api#/definitions/cloud_auth_method'
                     },
@@ -457,6 +475,9 @@ module.exports = {
                     },
                     endpoint_type: {
                         $ref: 'common_api#/definitions/endpoint_type'
+                    },
+                    ignore_name_already_exist: {
+                        type: 'boolean'
                     }
                 }
             },
