@@ -322,9 +322,10 @@ class ObjectSDK {
                 namespace_resource_id: ns_info.id,
                 rpc_client: this.rpc_client,
                 container: ns_info.target_bucket,
-                connection_string: cloud_utils.get_azure_connection_string(ns_info),
+                connection_string: cloud_utils.get_azure_new_connection_string(ns_info),
                 // Azure storage account name is stored as the access key.
-                account_name: ns_info.access_key.unwrap()
+                account_name: ns_info.access_key.unwrap(),
+                account_key: ns_info.secret_key.unwrap()
             });
         }
         if (ns_info.fs_root_path) {
