@@ -35,6 +35,7 @@ const events_dispatcher = require('./events_dispatcher');
 const { IoStatsStore } = require('../analytic_services/io_stats_store');
 const { ChunkAPI } = require('../../sdk/map_api_types');
 const config = require('../../../config');
+const Quota = require('../system_services/objects/quota');
 
 // short living cache for objects
 // the purpose is to reduce hitting the DB many many times per second during upload/download.
@@ -107,6 +108,10 @@ async function create_object_upload(req) {
         info.xattr = _.mapKeys(req.rpc_params.xattr, (v, k) => k.replace(/\./g, '@'));
     }
 
+    if (req.rpc_params.content_encoding) {
+        info.content_encoding = req.rpc_params.content_encoding;
+    }
+
     const tier = await map_server.select_tier_for_write(req.bucket);
     await MDStore.instance().insert_object(info);
     object_md_cache.put_in_cache(String(info._id), info);
@@ -1356,6 +1361,7 @@ function get_object_info(md, options = {}) {
         md5_b64: md.md5_b64 || undefined,
         sha256_b64: md.sha256_b64 || undefined,
         content_type: md.content_type || 'application/octet-stream',
+        content_encoding: md.content_encoding,
         create_time: md.create_time ? md.create_time.getTime() : md._id.getTimestamp().getTime(),
         last_modified_time: md.last_modified_time ? (md.last_modified_time.getTime()) : undefined,
         cache_last_valid_time: md.cache_last_valid_time ? md.cache_last_valid_time.getTime() : undefined,
@@ -1572,28 +1578,21 @@ function throw_if_maintenance(req) {
 function check_quota(bucket) {
     if (!bucket.quota) return;
 
-    let used_percent = system_utils.get_bucket_quota_usage_percent(bucket, bucket.quota);
+    const quota = new Quota(bucket.quota);
+    const alerts = [];
+    const major_messages = [];
+    quota.add_quota_alerts(system_store.data.systems[0]._id, bucket, alerts);
+    for (const alert of alerts) {
+        Dispatcher.instance().alert(alert.sev, alert.sysid, alert.alert, alert.rule);
+        if (alert.sev === 'MAJOR') {
+            major_messages.push(alert.alert);
+        }
+    }
 
-    if (used_percent >= 100) {
-        const bucket_used = bucket.storage_stats && size_utils.json_to_bigint(bucket.storage_stats.objects_size);
-        const message = `the bucket ${bucket.name} used storage(${
-                size_utils.human_size(bucket_used)
-            }) exceeds the bucket quota (${
-                size_utils.human_size(bucket.quota.value)
-            })`;
-        Dispatcher.instance().alert('MAJOR',
-            system_store.data.systems[0]._id,
-            `Bucket ${bucket.name.unwrap()} exceeded its configured quota of ${
-                    size_utils.human_size(bucket.quota.value)
-                }, uploads to this bucket will be denied`,
-            Dispatcher.rules.once_daily);
+    if (major_messages.length > 0) {
+        const message = major_messages.join();
         dbg.error(message);
         throw new RpcError('INVALID_BUCKET_STATE', message);
-    } else if (used_percent >= 90) {
-        Dispatcher.instance().alert('INFO',
-            system_store.data.systems[0]._id,
-            `Bucket ${bucket.name.unwrap()} exceeded 90% of its configured quota of ${size_utils.human_size(bucket.quota.value)}`,
-            Dispatcher.rules.once_daily);
     }
 }
 
@@ -1933,7 +1932,7 @@ async function _complete_object_multiparts(obj, multipart_req) {
                 `multipart num=${num} etag=${etag} etag_md5_b64=${etag_md5_b64} not found in group ${util.inspect(group)}`);
         }
         md5.update(Buffer.from(mp.md5_b64, 'base64'));
-        const mp_parts = parts_by_mp[mp._id.toHexString()];
+        const mp_parts = parts_by_mp[mp._id.toHexString()] || [];
         _complete_next_parts(mp_parts, context);
         used_multiparts.push(mp);
         for (const part of mp_parts) {
