@@ -30,8 +30,7 @@ const system_store = require('../system_services/system_store').get_instance();
 const func_store = require('../func_services/func_store');
 const replication_store = require('../system_services/replication_store');
 const node_allocator = require('../node_services/node_allocator');
-const system_utils = require('../utils/system_utils');
-const azure_storage = require('../../util/azure_storage_wrap');
+const azure_storage = require('../../util/new_azure_storage_wrap');
 const usage_aggregator = require('../bg_services/usage_aggregator');
 const chunk_config_utils = require('../utils/chunk_config_utils');
 const NetStorage = require('../../util/NetStorageKit-Node-master/lib/netstorage');
@@ -39,6 +38,7 @@ const { OP_NAME_TO_ACTION } = require('../../endpoint/s3/s3_utils');
 const path = require('path');
 const KeysSemaphore = require('../../util/keys_semaphore');
 const bucket_semaphore = new KeysSemaphore(1);
+const Quota = require('../system_services/objects/quota');
 
 const VALID_BUCKET_NAME_REGEXP =
     /^(([a-z0-9]|[a-z0-9][a-z0-9-]*[a-z0-9])\.)*([a-z0-9]|[a-z0-9][a-z0-9-]*[a-z0-9])$/;
@@ -643,7 +643,7 @@ function get_bucket_changes_namespace(req, bucket, update_request, single_bucket
     }
 }
 
-function get_bucket_changes_quota(req, bucket, quota, single_bucket_update, changes) {
+function get_bucket_changes_quota(req, bucket, quota_config, single_bucket_update, changes) {
     const quota_event = {
         event: 'bucket.quota',
         level: 'info',
@@ -652,33 +652,23 @@ function get_bucket_changes_quota(req, bucket, quota, single_bucket_update, chan
         bucket: bucket._id,
     };
 
-    if (quota === null) {
+    const quota = new Quota(quota_config);
+    if (quota.is_empty_quota()) {
         single_bucket_update.$unset = { quota: 1 };
         quota_event.desc = `Bucket quota was removed from ${bucket.name.unwrap()} by ${req.account && req.account.email.unwrap()}`;
     } else {
-        if (quota.size <= 0) throw new RpcError('BAD_REQUEST', 'quota size must be positive');
-        single_bucket_update.quota = quota;
-        quota.value = size_utils.size_unit_to_bigint(quota.size, quota.unit).toJSON();
-        let used_percent = system_utils.get_bucket_quota_usage_percent(bucket, quota);
-        if (used_percent >= 100) {
-            changes.alerts.push({
-                sev: 'MAJOR',
-                sysid: system_store.data.systems[0]._id,
-                alert: `Bucket ${bucket.name.unwrap()} exceeded its configured quota of ${
-                    size_utils.human_size(quota.value)
-                }, uploads to this bucket will be denied`,
-                rule: Dispatcher.rules.once_daily
-            });
-            dbg.warn(`the bucket ${bucket.name} used capacity is more than the updated quota. uploads will be denied`);
-        } else if (used_percent >= 90) {
-            changes.alerts.push({
-                sev: 'INFO',
-                sysid: system_store.data.systems[0]._id,
-                alert: `Bucket ${bucket.name.unwrap()} exceeded 90% of its configured quota of ${size_utils.human_size(quota.value)}`,
-                rule: Dispatcher.rules.once_daily
-            });
-        }
-        quota_event.desc = `Quota of ${size_utils.human_size(quota.value)} was set on ${bucket.name.unwrap()} by ${req.account && req.account.email.unwrap()}`;
+        if (!quota.is_valid_quota()) throw new RpcError('BAD_REQUEST', 'quota config values must be positive');
+
+        quota.add_quota_alerts(system_store.data.systems[0]._id, bucket, changes.alerts);
+
+        //Make event description
+        const quota_size_raw_value = quota.get_quota_by_size();
+        const quota_quantity_raw_data = quota.get_quota_by_quantity();
+        quota_event.desc = `Quota of ${quota_size_raw_value > 0 ? size_utils.human_size(quota_size_raw_value) : 'unlimited'} size
+        and ${quota_quantity_raw_data > 0 ? size_utils.human_size(quota_quantity_raw_data) : 'unlimited'} qunatity 
+        was set on ${bucket.name.unwrap()} by ${req.account && req.account.email.unwrap()}`;
+
+        single_bucket_update.quota = quota.get_config();
     }
     changes.events.push(quota_event);
 }
@@ -915,9 +905,10 @@ async function delete_bucket(req) {
  * DELETE_BUCKET_LIFECYCLE
  *
  */
-function delete_bucket_lifecycle(req) {
-    var bucket = find_bucket(req);
-    return system_store.make_changes({
+async function delete_bucket_lifecycle(req) {
+    const bucket = find_bucket(req);
+    try {
+        await system_store.make_changes({
             update: {
                 buckets: [{
                     _id: bucket._id,
@@ -926,24 +917,22 @@ function delete_bucket_lifecycle(req) {
                     }
                 }]
             }
-        })
-        .then(() => {
-            let desc_string = [];
-            desc_string.push(`lifecycle configuration rules were removed for bucket ${bucket.name.unwrap()} by ${req.account && req.account.email.unwrap()}`);
-
-            Dispatcher.instance().activity({
-                event: 'bucket.delete_lifecycle_configuration_rules',
-                level: 'info',
-                system: req.system._id,
-                actor: req.account && req.account._id,
-                bucket: bucket._id,
-                desc: desc_string.join('\n'),
-            });
-        })
-        .catch(function(err) {
-            dbg.error('Error deleting lifecycle configuration rules', err, err.stack);
-            throw err;
         });
+        const desc_string = [];
+        desc_string.push(`lifecycle configuration rules were removed for bucket ${bucket.name.unwrap()} by ${req.account && req.account.email.unwrap()}`);
+
+        Dispatcher.instance().activity({
+            event: 'bucket.delete_lifecycle_configuration_rules',
+            level: 'info',
+            system: req.system._id,
+            actor: req.account && req.account._id,
+            bucket: bucket._id,
+            desc: desc_string.join('\n'),
+        });
+    } catch (err) {
+        dbg.error('Error deleting lifecycle configuration rules', err, err.stack);
+        throw err;
+    }
 }
 
 /**
@@ -1047,38 +1036,37 @@ async function get_buckets_stats_by_content_type(req) {
  * SET_BUCKET_LIFECYCLE_CONFIGURATION_RULES
  *
  */
-function set_bucket_lifecycle_configuration_rules(req) {
+async function set_bucket_lifecycle_configuration_rules(req) {
     dbg.log0('set bucket lifecycle configuration rules', req.rpc_params);
 
-    var bucket = find_bucket(req);
+    const bucket = find_bucket(req);
 
-    var lifecycle_configuration_rules = req.rpc_params.rules;
+    const lifecycle_configuration_rules = req.rpc_params.rules;
 
-    return system_store.make_changes({
+    try {
+        await system_store.make_changes({
             update: {
                 buckets: [{
                     _id: bucket._id,
                     lifecycle_configuration_rules: lifecycle_configuration_rules
                 }]
             }
-        })
-        .then(() => {
-            let desc_string = [];
-            desc_string.push(`${bucket.name.unwrap()} was updated with lifecycle configuration rules by ${req.account && req.account.email.unwrap()}`);
-
-            Dispatcher.instance().activity({
-                event: 'bucket.set_lifecycle_configuration_rules',
-                level: 'info',
-                system: req.system._id,
-                actor: req.account && req.account._id,
-                bucket: bucket._id,
-                desc: desc_string.join('\n'),
-            });
-        })
-        .catch(function(err) {
-            dbg.error('Error setting lifecycle configuration rules', err, err.stack);
-            throw err;
         });
+        const desc_string = [];
+        desc_string.push(`${bucket.name.unwrap()} was updated with lifecycle configuration rules by ${req.account && req.account.email.unwrap()}`);
+
+        Dispatcher.instance().activity({
+            event: 'bucket.set_lifecycle_configuration_rules',
+            level: 'info',
+            system: req.system._id,
+            actor: req.account && req.account._id,
+            bucket: bucket._id,
+            desc: desc_string.join('\n'),
+        });
+    } catch (err) {
+        dbg.error('Error setting lifecycle configuration rules', err, err.stack);
+        throw err;
+    }
 }
 
 /**
@@ -1088,7 +1076,7 @@ function set_bucket_lifecycle_configuration_rules(req) {
  */
 function get_bucket_lifecycle_configuration_rules(req) {
     dbg.log0('get bucket lifecycle configuration rules', req.rpc_params);
-    var bucket = find_bucket(req);
+    const bucket = find_bucket(req);
     return bucket.lifecycle_configuration_rules || [];
 }
 
@@ -1099,21 +1087,23 @@ function get_bucket_lifecycle_configuration_rules(req) {
  */
 function get_cloud_buckets(req) {
     dbg.log0('get cloud buckets', req.rpc_params);
-    return P.fcall(function() {
+    return P.fcall(async function() {
             var connection = cloud_utils.find_cloud_connection(
                 req.account,
                 req.rpc_params.connection
             );
             if (connection.endpoint_type === 'AZURE') {
-                let blob_svc = azure_storage.createBlobService(cloud_utils.get_azure_connection_string(connection));
-                let used_cloud_buckets = cloud_utils.get_used_cloud_targets(['AZURE'],
+                const blob_svc = azure_storage.BlobServiceClient.fromConnectionString(
+                    cloud_utils.get_azure_new_connection_string(connection));
+                const used_cloud_buckets = cloud_utils.get_used_cloud_targets(['AZURE'],
                     system_store.data.buckets, system_store.data.pools, system_store.data.namespace_resources);
-                return P.timeout(EXTERNAL_BUCKET_LIST_TO, P.fromCallback(callback => {
-                        blob_svc.listContainersSegmented(null, { maxResults: 100 }, callback);
-                    }))
-                    .then(data => data.entries.map(entry =>
-                        _inject_usage_to_cloud_bucket(entry.name, connection.endpoint, used_cloud_buckets)
-                    ));
+                return P.timeout(EXTERNAL_BUCKET_LIST_TO,
+                        blob_svc.listContainers().byPage({ maxPageSize: 100 }).next())
+                    .then(iterator => {
+                        const response = iterator.value;
+                        return response.containerItems.map(entry =>
+                            _inject_usage_to_cloud_bucket(entry.name, connection.endpoint, used_cloud_buckets));
+                    });
             } else if (connection.endpoint_type === 'NET_STORAGE') {
                 let used_cloud_buckets = cloud_utils.get_used_cloud_targets(['NET_STORAGE'],
                     system_store.data.buckets, system_store.data.pools, system_store.data.namespace_resources);
@@ -1154,18 +1144,30 @@ function get_cloud_buckets(req) {
                 return storage.getBuckets()
                     .then(data => data[0].map(bucket =>
                         _inject_usage_to_cloud_bucket(bucket.name, connection.endpoint, used_cloud_buckets)));
-            } else { //else if AWS
+            } else { // else if AWS(s3-compatible/aws/sts-aws)/Flashblade/IBM_COS
+                var access_key;
+                var secret_key;
+                if (connection.aws_sts_arn) {
+                    const creds = await cloud_utils.generate_aws_sts_creds(connection, "get_cloud_buckets_session");
+                    access_key = creds.accessKeyId;
+                    secret_key = creds.secretAccessKey;
+                    connection.sessionToken = creds.sessionToken;
+                } else {
+                    access_key = connection.access_key.unwrap();
+                    secret_key = connection.secret_key.unwrap();
+                }
                 var s3 = new AWS.S3({
                     endpoint: connection.endpoint,
-                    accessKeyId: connection.access_key.unwrap(),
-                    secretAccessKey: connection.secret_key.unwrap(),
+                    accessKeyId: access_key,
+                    secretAccessKey: secret_key,
+                    sessionToken: connection.sessionToken,
                     signatureVersion: cloud_utils.get_s3_endpoint_signature_ver(connection.endpoint, connection.auth_method),
                     s3DisableBodySigning: cloud_utils.disable_s3_compatible_bodysigning(connection.endpoint),
                     httpOptions: {
                         agent: http_utils.get_unsecured_agent(connection.endpoint)
                     }
                 });
-                const used_cloud_buckets = cloud_utils.get_used_cloud_targets(['AWS', 'S3_COMPATIBLE', 'FLASHBLADE', 'IBM_COS'],
+                const used_cloud_buckets = cloud_utils.get_used_cloud_targets(['AWS', 'AWS_STS', 'S3_COMPATIBLE', 'FLASHBLADE', 'IBM_COS'],
                     system_store.data.buckets, system_store.data.pools, system_store.data.namespace_resources);
                 return P.timeout(EXTERNAL_BUCKET_LIST_TO, P.ninvoke(s3, 'listBuckets'))
                     .then(data => data.Buckets.map(bucket =>
@@ -1634,13 +1636,11 @@ function _calc_metrics({
     let is_quota_exceeded = false;
     let is_quota_low = false;
     if (bucket.quota) {
-        let quota_precent = system_utils.get_bucket_quota_usage_percent(bucket, bucket.quota);
-        info.quota = _.omit(bucket.quota, 'value');
-        if (quota_precent >= 100) {
-            is_quota_exceeded = true;
-        } else if (quota_precent >= 90) {
-            is_quota_low = true;
-        }
+        const quota = new Quota(bucket.quota);
+        info.quota = quota.get_config();
+        const quota_exceeded = quota.is_quota_exceeded(bucket);
+        is_quota_exceeded = quota_exceeded.is_quota_exceeded;
+        is_quota_low = quota_exceeded.is_quota_low;
     }
     let risky_tolerance = false;
 
@@ -1694,15 +1694,10 @@ function _calc_metrics({
     };
 
     const actual_free = size_utils.json_to_bigint(_.get(info, 'tiering.data.free') || 0);
-    let available_for_upload = actual_free;
 
-    if (bucket.quota) {
-        let quota_free = size_utils.json_to_bigint(bucket.quota.value).minus(size_utils.json_to_bigint(objects_aggregate.size));
-        available_for_upload = size_utils.size_min([
-            size_utils.bigint_to_json(quota_free),
-            size_utils.bigint_to_json(available_for_upload)
-        ]);
-    }
+    const quota = new Quota(bucket.quota);
+    let available_size_for_upload = quota.get_available_size_for_upload(actual_free, objects_aggregate.size);
+    let available_quantity_for_upload = quota.get_available_quantity_for_upload(objects_aggregate.count);
 
     if (bucket_free.isZero()) {
         is_no_storage = true;
@@ -1718,7 +1713,8 @@ function _calc_metrics({
         size: objects_aggregate.size,
         size_reduced: bucket_chunks_capacity,
         free: actual_free,
-        available_for_upload,
+        available_size_for_upload,
+        available_quantity_for_upload,
         last_update,
     });
 
