@@ -2,7 +2,7 @@
     typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
     typeof define === 'function' && define.amd ? define(factory) :
     (global = typeof globalThis !== 'undefined' ? globalThis : global || self, (global.date = global.date || {}, global.date.locale = global.date.locale || {}, global.date.locale.en = factory()));
-}(this, (function () { 'use strict';
+})(this, (function () { 'use strict';
 
     /**
      * @preserve date-and-time.js locale configuration
@@ -10,11 +10,12 @@
      * @preserve This is a dummy module.
      */
 
-    var en = function () {
+    var en = function (date) {
         var code = 'en';
+
         return code;
     };
 
     return en;
 
-})));
+}));
